#!/usr/bin/env python3

import sys
import os
import argparse
import pkg_resources
from collections import Counter

import pysam

from bammix import __version__
from .bammix_functions import *

thisdir = os.path.abspath(os.path.dirname(__file__))
cwd = os.getcwd()

def main(sysargs = sys.argv[1:]):

    parser = argparse.ArgumentParser(#prog = _program,
    description = "bammix",
    usage = '''bammix -b bam_file.bam -p positions [options]''')

    parser.add_argument("-b", "--bam", help = "BAM file to be examined")
    parser.add_argument("-p", "--positions", nargs = "+", help = "Set of positions to be checked")
    parser.add_argument("-r", "--reference", help = "Name of reference the reads were mapped against, will be extracted from the BAM if not provided", default = None)
    parser.add_argument("-q", "--base_quality", help = "Minimum base quality to include a base in counts, default is to include all bases regardless of quality", default = "0")
    parser.add_argument("-m", "--mapping_quality", help = "Minimum mapping quality to include a read in counts, default is to include all reads", default = "0")
    parser.add_argument("-o", "--output_prefix", help = "Optional prefix for output files, default is no prefix", default = None)

    if len(sysargs)<1:
        parser.print_help()
        sys.exit(-1)
    else:
        args = parser.parse_args(sysargs)
    
    #Import BAM
    bam = pysam.Samfile(args.bam, "rb")

    #Minimum base quality
    bq = int(args.base_quality)
    #Minimum mapping quality
    mq = int(args.mapping_quality)

    #Reference sequence name
    if args.reference:
        reference = args.reference
    else:
        reference = bam.get_reference_name(0)

    if args.output_prefix:
        outcsv = os.path.join(cwd + "/" + args.output_prefix + "_position_base_counts.csv")
    else:
        outcsv = os.path.join(cwd + "/" + "position_base_counts.csv")
    
    outFile = open(outcsv, "w")
    outFile.write("Position,Total_reads,A_reads,C_reads,G_reads,T_reads,A_proportion,C_proportion,G_proportion,T_proportion\n")

    positions, read_depth, nA, nC, nG, nT = extract_nucleotides(bam, args.positions, reference, bq, mq, outFile)
    
    outFile.close()

    if args.output_prefix:
        outPlot = os.path.join(cwd + "/" + args.output_prefix + "_position_base_counts.pdf")
    else:
        outPlot = os.path.join(cwd + "/" + "position_base_counts.pdf")

    plot_nucleotides(positions, read_depth, nA, nC, nG, nT, outPlot)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# imports of built-ins
import os
import sys
import argparse
from collections import Counter

import pysam
import matplotlib.pyplot as plt

#Extracts all of the nucleotides at a given position
def get_bases(bam, position, reference, bq, mq):
    bases = ""

    for c in bam.pileup(reference, (position - 1), position, min_base_quality = bq):
        for r in c.pileups:
            #Check if read mapping quality is high enough to include
            if r.alignment.mapping_quality >= mq:
                if c.pos == (position - 1):
                    if r.query_position is not None:
                        bases += r.alignment.query_sequence[r.query_position]
    
    return(bases)

#Counts the number and calculates the proportion of each nucleotide at each of a given set of positions
def extract_nucleotides(bam, pos, reference, bq, mq, outFile):

    #Lists for plotting
    positions = list()
    read_depth = list()
    nA = list()
    nC = list()
    nG = list()
    nT = list()

    #Iterate through the positions, identify the nucleotide in each read and count each base
    for p in pos:
        position = int(p)

        bases = get_bases(bam, position, reference, bq, mq)

        bc = Counter(bases)

        tr = float(sum(bc.values()))

        outFile.write(p + "," + str(tr) + ",")
        if "A" in bc:
            outFile.write(str(bc["A"]) + ",")
        else:
            outFile.write("0,")
        if "C" in bc:
            outFile.write(str(bc["C"]) + ",")
        else:
            outFile.write("0,")
        if "G" in bc:
            outFile.write(str(bc["G"]) + ",")
        else:
            outFile.write("0,")
        if "T" in bc:
            outFile.write(str(bc["T"]) + ",")
        else:
            outFile.write("0,")
        
        if "A" in bc:
            outFile.write(str(float(bc["A"])/tr) + ",")
            nA.append(float(bc["A"])/tr)
        else:
            outFile.write("0,")
            nA.append(float(0))
        if "C" in bc:
            outFile.write(str(float(bc["C"])/tr) + ",")
            nC.append(float(bc["C"])/tr)
        else:
            outFile.write("0,")
            nC.append(float(0))
        if "G" in bc:
            outFile.write(str(float(bc["G"])/tr) + ",")
            nG.append(float(bc["G"])/tr)
        else:
            outFile.write("0,")
            nG.append(float(0))
        if "T" in bc:
            outFile.write(str(float(bc["T"])/tr) + "\n")
            nT.append(float(bc["T"])/tr)
        else:
            outFile.write("0\n")
            nT.append(float(0))
        
        #Append to plot lists
        positions.append(p)
        read_depth.append(tr)
    
    return(positions, read_depth, nA, nC, nG, nT)

#Plots the proportion of each nucleotide at each position
def plot_nucleotides(positions, read_depth, nA, nC, nG, nT, outPlot):
    fig, ax = plt.subplots()

    #Plot nucleotide proportions
    ax.bar(positions, nA, label = "A", color = "red")
    ax.bar(positions, nC, label = "C", bottom = nA, color = "blue")
    ax.bar(positions, nG, label = "G", bottom = [x + y for x, y in zip(nA, nC)], color = "purple")
    ax.bar(positions, nT, label = "T", bottom = [x + y + z for x, y, z in zip(nA, nC, nG)], color = "orange")

    #Add read depth labels
    for i in range(len(positions)):
        ax.text(i, 1.02, int(read_depth[i]), ha = "center", rotation = "vertical")

    #Read depth label
    if len(positions) <= 5:
        ax.text(-1.5, 1.05, "Read depth:")
    elif len(positions) <= 10:
        ax.text(-2.5, 1.05, "Read depth:")
    else:
        ax.text(-3.5, 1.05, "Read depth:")
    ax.set_xticklabels(positions, rotation = 90)
    ax.set_xlabel("Position")
    ax.set_ylabel("Nucleotide proportion")
    ax.legend(bbox_to_anchor = (1.04, 0.6))

    plt.savefig(outPlot, bbox_inches = "tight")


from setuptools import setup, find_packages
from codecs import open
from os import path
import os
import re
import io

# Get version strip
def read(*names, **kwargs):
    with io.open(os.path.join(os.path.dirname(__file__), *names),
                 encoding=kwargs.get("encoding", "utf8")) as fp:
        return fp.read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

#Import README
here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(name='bammix',
      version=find_version("bammix/__init__.py"),
      author='Chris Ruis',
      author_email='cr628@cam.ac.uk',
      description='bammix',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/chrisruis/bammix',
      install_requires=[
            "pysam>=0.17.0",
            "matplotlib>=3.2.1"
        ],
      python_requires=">=3.6.0",
      packages=['bammix'],
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: MIT License",
          "Operating System :: OS Independent",
        ],
      entry_points={
          'console_scripts': [
              'bammix = bammix.command:main',
            ],
          },
      )
